
#include "qmi8658c.h"

I2C_HandleTypeDef hi2c1;  // I2C handle from STM32 HAL

void IMU_Init(void)
{
    QMI8658C_Config_t config;
    
    // Configure the IMU
    config.accelRange = QMI8658C_ACCEL_RANGE_4G;
    config.accelODR = QMI8658C_ACCEL_ODR_125HZ;
    config.gyroRange = QMI8658C_GYRO_RANGE_128DPS;
    config.gyroODR = QMI8658C_GYRO_ODR_125HZ;
    config.magDevice = QMI8658C_MAG_NONE;
    config.magODR = QMI8658C_MAG_ODR_125HZ;
    config.accelLPFMode = QMI8658C_LPF_MODE_2;
    config.gyroLPFMode = QMI8658C_LPF_MODE_2;
    config.accelLPFEnabled = true;
    config.gyroLPFEnabled = true;
    config.aeEnabled = false;
    config.magEnabled = false;
    config.gyroEnabled = true;
    config.accelEnabled = true;
    config.fifoMode = QMI8658C_FIFO_MODE_BYPASS;
    
    // Initialize the IMU with custom configuration
    if (QMI8658C_InitWithConfig(&hi2c1, &config) != QMI8658C_OK) {
        // Handle error
        Error_Handler();
    }
}

void IMU_ReadData(void)
{
    QMI8658C_IMUData_t imuData;
    
    // Read IMU data
    if (QMI8658C_ReadIMUData(&imuData) == QMI8658C_OK) {
        float accelX = (float)imuData.ax / 8192.0f;  // Convert to g (±4g range)
        float accelY = (float)imuData.ay / 8192.0f;
        float accelZ = (float)imuData.az / 8192.0f;
        
        float gyroX = (float)imuData.gx / 256.0f;   // Convert to dps (±128dps range)
        float gyroY = (float)imuData.gy / 256.0f;
        float gyroZ = (float)imuData.gz / 256.0f;
        
        float tempC = (float)imuData.temp / 256.0f; // Convert to degrees Celsius
        
        // Process the data as needed
    }
}