/**
 * @file qmi8658c.h
 * @brief Header file for QMI8658C IMU driver
 * @author Generated
 * @date 2023
 */

#ifndef QMI8658C_H
#define QMI8658C_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdbool.h>

/* Device Information */
#define QMI8658C_WHO_AM_I         0x05
#define QMI8658C_REVISION_ID      0x79

/* I2C Address */
#define QMI8658C_I2C_ADDR         0x6A  // 0x6A (default with SA0=0), 0x6B (with SA0=1)

/* Register Addresses */
#define QMI8658C_REG_WHO_AM_I     0x00
#define QMI8658C_REG_REVISION_ID  0x01
#define QMI8658C_REG_CTRL1        0x02
#define QMI8658C_REG_CTRL2        0x03
#define QMI8658C_REG_CTRL3        0x04
#define QMI8658C_REG_CTRL4        0x05
#define QMI8658C_REG_CTRL5        0x06
#define QMI8658C_REG_CTRL6        0x07
#define QMI8658C_REG_CTRL7        0x08
#define QMI8658C_REG_CTRL8        0x09
#define QMI8658C_REG_CTRL9        0x0A

/* Calibration Registers */
#define QMI8658C_REG_CAL1_L       0x0B
#define QMI8658C_REG_CAL1_H       0x0C
#define QMI8658C_REG_CAL2_L       0x0D
#define QMI8658C_REG_CAL2_H       0x0E
#define QMI8658C_REG_CAL3_L       0x0F
#define QMI8658C_REG_CAL3_H       0x10
#define QMI8658C_REG_CAL4_L       0x11
#define QMI8658C_REG_CAL4_H       0x12

/* FIFO Registers */
#define QMI8658C_REG_FIFO_WTM_TH  0x13
#define QMI8658C_REG_FIFO_CTRL    0x14
#define QMI8658C_REG_FIFO_SMPL_CNT 0x15
#define QMI8658C_REG_FIFO_STATUS  0x16
#define QMI8658C_REG_FIFO_DATA    0x17

/* Status Registers */
#define QMI8658C_REG_I2CM_STATUS  0x2C
#define QMI8658C_REG_STATUSINT    0x2D
#define QMI8658C_REG_STATUS0      0x2E
#define QMI8658C_REG_STATUS1      0x2F

/* Timestamp Registers */
#define QMI8658C_REG_TIMESTAMP_LOW  0x30
#define QMI8658C_REG_TIMESTAMP_MID  0x31
#define QMI8658C_REG_TIMESTAMP_HIGH 0x32

/* Data Output Registers */
#define QMI8658C_REG_TEMP_L       0x33
#define QMI8658C_REG_TEMP_H       0x34
#define QMI8658C_REG_AX_L         0x35
#define QMI8658C_REG_AX_H         0x36
#define QMI8658C_REG_AY_L         0x37
#define QMI8658C_REG_AY_H         0x38
#define QMI8658C_REG_AZ_L         0x39
#define QMI8658C_REG_AZ_H         0x3A
#define QMI8658C_REG_GX_L         0x3B
#define QMI8658C_REG_GX_H         0x3C
#define QMI8658C_REG_GY_L         0x3D
#define QMI8658C_REG_GY_H         0x3E
#define QMI8658C_REG_GZ_L         0x3F
#define QMI8658C_REG_GZ_H         0x40
#define QMI8658C_REG_MX_L         0x41
#define QMI8658C_REG_MX_H         0x42
#define QMI8658C_REG_MY_L         0x43
#define QMI8658C_REG_MY_H         0x44
#define QMI8658C_REG_MZ_L         0x45
#define QMI8658C_REG_MZ_H         0x46

/* AttitudeEngine Output Registers */
#define QMI8658C_REG_DQW_L        0x49
#define QMI8658C_REG_DQW_H        0x4A
#define QMI8658C_REG_DQX_L        0x4B
#define QMI8658C_REG_DQX_H        0x4C
#define QMI8658C_REG_DQY_L        0x4D
#define QMI8658C_REG_DQY_H        0x4E
#define QMI8658C_REG_DQZ_L        0x4F
#define QMI8658C_REG_DQZ_H        0x50
#define QMI8658C_REG_DVX_L        0x51
#define QMI8658C_REG_DVX_H        0x52
#define QMI8658C_REG_DVY_L        0x53
#define QMI8658C_REG_DVY_H        0x54
#define QMI8658C_REG_DVZ_L        0x55
#define QMI8658C_REG_DVZ_H        0x56
#define QMI8658C_REG_AE_REG1      0x57
#define QMI8658C_REG_AE_REG2      0x58

/* Reset Register */
#define QMI8658C_REG_RESET        0x60

/* CTRL9 Commands */
#define QMI8658C_CTRL_CMD_NOP                     0x00
#define QMI8658C_CTRL_CMD_GYRO_BIAS               0x01
#define QMI8658C_CTRL_CMD_REQ_SDI                 0x03
#define QMI8658C_CTRL_CMD_RST_FIFO                0x04
#define QMI8658C_CTRL_CMD_REQ_FIFO                0x05
#define QMI8658C_CTRL_CMD_I2CM_WRITE              0x06
#define QMI8658C_CTRL_CMD_WRITE_WOM_SETTING       0x08
#define QMI8658C_CTRL_CMD_ACCEL_HOST_DELTA_OFFSET 0x09
#define QMI8658C_CTRL_CMD_GYRO_HOST_DELTA_OFFSET  0x0A
#define QMI8658C_CTRL_CMD_COPY_USID               0x10
#define QMI8658C_CTRL_CMD_SET_RPU                 0x11

/* CTRL1 Register Bit Definitions */
#define QMI8658C_CTRL1_SIM        (1 << 7)  // SPI Interface Mode (0: 4-wire, 1: 3-wire)
#define QMI8658C_CTRL1_SPI_AI     (1 << 6)  // Address Increment (0: disabled, 1: enabled)
#define QMI8658C_CTRL1_SPI_BE     (1 << 5)  // Endian (0: little endian, 1: big endian)
#define QMI8658C_CTRL1_SENSOR_DISABLE (1 << 0)  // Sensor disable (0: enabled, 1: disabled)

/* CTRL2 Accelerometer Settings */
/* Accelerometer Output Data Rate */
typedef enum {
    QMI8658C_ACCEL_ODR_8000HZ  = 0x00,
    QMI8658C_ACCEL_ODR_4000HZ  = 0x01,
    QMI8658C_ACCEL_ODR_2000HZ  = 0x02,
    QMI8658C_ACCEL_ODR_1000HZ  = 0x03,
    QMI8658C_ACCEL_ODR_500HZ   = 0x04,
    QMI8658C_ACCEL_ODR_250HZ   = 0x05,
    QMI8658C_ACCEL_ODR_125HZ   = 0x06,
    QMI8658C_ACCEL_ODR_62_5HZ  = 0x07,
    QMI8658C_ACCEL_ODR_31_25HZ = 0x08,
    QMI8658C_ACCEL_ODR_128HZ_LP = 0x0C,
    QMI8658C_ACCEL_ODR_21HZ_LP = 0x0D,
    QMI8658C_ACCEL_ODR_11HZ_LP = 0x0E,
    QMI8658C_ACCEL_ODR_3HZ_LP  = 0x0F
} QMI8658C_AccelODR_t;

/* Accelerometer Full Scale */
typedef enum {
    QMI8658C_ACCEL_RANGE_2G  = 0x00,
    QMI8658C_ACCEL_RANGE_4G  = 0x01,
    QMI8658C_ACCEL_RANGE_8G  = 0x02,
    QMI8658C_ACCEL_RANGE_16G = 0x03
} QMI8658C_AccelRange_t;

/* CTRL3 Gyroscope Settings */
/* Gyroscope Output Data Rate */
typedef enum {
    QMI8658C_GYRO_ODR_8000HZ  = 0x00,
    QMI8658C_GYRO_ODR_4000HZ  = 0x01,
    QMI8658C_GYRO_ODR_2000HZ  = 0x02,
    QMI8658C_GYRO_ODR_1000HZ  = 0x03,
    QMI8658C_GYRO_ODR_500HZ   = 0x04,
    QMI8658C_GYRO_ODR_250HZ   = 0x05,
    QMI8658C_GYRO_ODR_125HZ   = 0x06,
    QMI8658C_GYRO_ODR_62_5HZ  = 0x07,
    QMI8658C_GYRO_ODR_31_25HZ = 0x08
} QMI8658C_GyroODR_t;

/* Gyroscope Full Scale */
typedef enum {
    QMI8658C_GYRO_RANGE_16DPS   = 0x00,
    QMI8658C_GYRO_RANGE_32DPS   = 0x01,
    QMI8658C_GYRO_RANGE_64DPS   = 0x02,
    QMI8658C_GYRO_RANGE_128DPS  = 0x03,
    QMI8658C_GYRO_RANGE_256DPS  = 0x04,
    QMI8658C_GYRO_RANGE_512DPS  = 0x05,
    QMI8658C_GYRO_RANGE_1024DPS = 0x06,
    QMI8658C_GYRO_RANGE_2048DPS = 0x07
} QMI8658C_GyroRange_t;

/* CTRL4 Magnetometer Settings */
/* Magnetometer Device Type */
typedef enum {
    QMI8658C_MAG_NONE     = 0x00,
    QMI8658C_MAG_AK09915C = 0x08,
    QMI8658C_MAG_AK09918CZ = 0x0C,
    QMI8658C_MAG_QMC6308  = 0x40
} QMI8658C_MagDevice_t;

/* Magnetometer Output Data Rate */
typedef enum {
    QMI8658C_MAG_ODR_1000HZ = 0x00,
    QMI8658C_MAG_ODR_500HZ  = 0x01,
    QMI8658C_MAG_ODR_250HZ  = 0x02,
    QMI8658C_MAG_ODR_125HZ  = 0x03,
    QMI8658C_MAG_ODR_62_5HZ = 0x04,
    QMI8658C_MAG_ODR_31_25HZ = 0x05
} QMI8658C_MagODR_t;

/* CTRL5 Filter Settings */
/* Low-Pass Filter Mode */
typedef enum {
    QMI8658C_LPF_MODE_0 = 0x00,  // 2.62% of ODR
    QMI8658C_LPF_MODE_1 = 0x01,  // 3.59% of ODR
    QMI8658C_LPF_MODE_2 = 0x02,  // 5.32% of ODR
    QMI8658C_LPF_MODE_3 = 0x03   // 14.0% of ODR
} QMI8658C_LPFMode_t;

/* CTRL6 AttitudeEngine Settings */
/* AttitudeEngine ODR */
typedef enum {
    QMI8658C_AE_ODR_1HZ  = 0x00,
    QMI8658C_AE_ODR_2HZ  = 0x01,
    QMI8658C_AE_ODR_4HZ  = 0x02,
    QMI8658C_AE_ODR_8HZ  = 0x03,
    QMI8658C_AE_ODR_16HZ = 0x04,
    QMI8658C_AE_ODR_32HZ = 0x05,
    QMI8658C_AE_ODR_64HZ = 0x06
} QMI8658C_AE_ODR_t;

/* CTRL7 Sensor Enable Settings */
#define QMI8658C_CTRL7_SYNC_SMPL (1 << 7)  // Synchronous Sample Mode
#define QMI8658C_CTRL7_SYS_HS    (1 << 6)  // High-Speed Internal Clock
#define QMI8658C_CTRL7_G_SNOOZE  (1 << 4)  // Gyroscope Snooze Mode
#define QMI8658C_CTRL7_AE_EN     (1 << 3)  // AttitudeEngine Enable
#define QMI8658C_CTRL7_MAG_EN    (1 << 2)  // Magnetometer Enable
#define QMI8658C_CTRL7_GYRO_EN   (1 << 1)  // Gyroscope Enable
#define QMI8658C_CTRL7_ACCEL_EN  (1 << 0)  // Accelerometer Enable

/* FIFO Control Register */
#define QMI8658C_FIFO_RD_MODE    (1 << 7)  // FIFO Read Mode

/* FIFO Size Settings */
typedef enum {
    QMI8658C_FIFO_SIZE_16  = 0x00,
    QMI8658C_FIFO_SIZE_32  = 0x01,
    QMI8658C_FIFO_SIZE_64  = 0x02,
    QMI8658C_FIFO_SIZE_128 = 0x03
} QMI8658C_FifoSize_t;

/* FIFO Mode Settings */
typedef enum {
    QMI8658C_FIFO_MODE_BYPASS   = 0x00,
    QMI8658C_FIFO_MODE_FIFO     = 0x01,
    QMI8658C_FIFO_MODE_STREAM   = 0x02,
    QMI8658C_FIFO_MODE_RESERVED = 0x03
} QMI8658C_FifoMode_t;

/* STATUS0 Register Bit Definitions */
#define QMI8658C_STATUS0_AE_DA  (1 << 3)  // AttitudeEngine Data Available
#define QMI8658C_STATUS0_MAG_DA (1 << 2)  // Magnetometer Data Available
#define QMI8658C_STATUS0_GYRO_DA (1 << 1) // Gyroscope Data Available
#define QMI8658C_STATUS0_ACCEL_DA (1 << 0) // Accelerometer Data Available

/* AttitudeEngine Device Orientation */
typedef struct {
    int16_t dQW;  // Quaternion W component
    int16_t dQX;  // Quaternion X component
    int16_t dQY;  // Quaternion Y component
    int16_t dQZ;  // Quaternion Z component
    int16_t dVX;  // Velocity X component
    int16_t dVY;  // Velocity Y component
    int16_t dVZ;  // Velocity Z component
} QMI8658C_AttitudeEngine_t;

/* 6-axis IMU Data */
typedef struct {
    int16_t ax;  // Accelerometer X-axis
    int16_t ay;  // Accelerometer Y-axis
    int16_t az;  // Accelerometer Z-axis
    int16_t gx;  // Gyroscope X-axis
    int16_t gy;  // Gyroscope Y-axis
    int16_t gz;  // Gyroscope Z-axis
    int16_t temp; // Temperature
} QMI8658C_IMUData_t;

/* 9-axis Data (IMU + Magnetometer) */
typedef struct {
    int16_t ax;  // Accelerometer X-axis
    int16_t ay;  // Accelerometer Y-axis
    int16_t az;  // Accelerometer Z-axis
    int16_t gx;  // Gyroscope X-axis
    int16_t gy;  // Gyroscope Y-axis
    int16_t gz;  // Gyroscope Z-axis
    int16_t mx;  // Magnetometer X-axis
    int16_t my;  // Magnetometer Y-axis
    int16_t mz;  // Magnetometer Z-axis
    int16_t temp; // Temperature
} QMI8658C_9DOFData_t;

/* Configuration Structure */
typedef struct {
    QMI8658C_AccelRange_t accelRange;
    QMI8658C_AccelODR_t accelODR;
    QMI8658C_GyroRange_t gyroRange;
    QMI8658C_GyroODR_t gyroODR;
    QMI8658C_MagDevice_t magDevice;
    QMI8658C_MagODR_t magODR;
    QMI8658C_LPFMode_t accelLPFMode;
    QMI8658C_LPFMode_t gyroLPFMode;
    bool accelLPFEnabled;
    bool gyroLPFEnabled;
    QMI8658C_AE_ODR_t aeODR;
    bool aeMoDEnabled;
    bool syncSampleEnabled;
    bool highSpeedClock;
    bool gyroSnoozeEnabled;
    bool aeEnabled;
    bool magEnabled;
    bool gyroEnabled;
    bool accelEnabled;
    QMI8658C_FifoSize_t fifoSize;
    QMI8658C_FifoMode_t fifoMode;
    uint8_t fifoWatermark;
} QMI8658C_Config_t;

/* QMI8658C Initialization Status */
typedef enum {
    QMI8658C_OK = 0,
    QMI8658C_ERROR,
    QMI8658C_ERROR_COMM,
    QMI8658C_ERROR_ID,
    QMI8658C_ERROR_INIT
} QMI8658C_Status_t;

/* Prototypes */

/**
 * @brief Initialize QMI8658C device with default configuration
 * 
 * @param hi2c Pointer to I2C handle structure 
 * @return QMI8658C_Status_t Initialization status
 */
QMI8658C_Status_t QMI8658C_Init(void *hi2c);

/**
 * @brief Initialize QMI8658C device with provided configuration
 * 
 * @param hi2c Pointer to I2C handle structure
 * @param config Pointer to configuration structure
 * @return QMI8658C_Status_t Initialization status
 */
QMI8658C_Status_t QMI8658C_InitWithConfig(void *hi2c, QMI8658C_Config_t *config);

/**
 * @brief Reset QMI8658C device
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_Reset(void);

/**
 * @brief Set QMI8658C device to default configuration
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_SetDefaultConfig(void);

/**
 * @brief Read QMI8658C device ID (WHO_AM_I register)
 * 
 * @param id Pointer to store device ID
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ReadID(uint8_t *id);

/**
 * @brief Enable accelerometer
 * 
 * @param odr Accelerometer output data rate
 * @param range Accelerometer range
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_EnableAccel(QMI8658C_AccelODR_t odr, QMI8658C_AccelRange_t range);

/**
 * @brief Disable accelerometer
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_DisableAccel(void);

/**
 * @brief Enable gyroscope
 * 
 * @param odr Gyroscope output data rate
 * @param range Gyroscope range
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_EnableGyro(QMI8658C_GyroODR_t odr, QMI8658C_GyroRange_t range);

/**
 * @brief Disable gyroscope
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_DisableGyro(void);

/**
 * @brief Enable magnetometer
 * 
 * @param device Magnetometer device type
 * @param odr Magnetometer output data rate
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_EnableMag(QMI8658C_MagDevice_t device, QMI8658C_MagODR_t odr);

/**
 * @brief Disable magnetometer
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_DisableMag(void);

/**
 * @brief Enable AttitudeEngine
 * 
 * @param odr AttitudeEngine output data rate
 * @param enableMoD Enable Motion on Demand mode
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_EnableAttitudeEngine(QMI8658C_AE_ODR_t odr, bool enableMoD);

/**
 * @brief Disable AttitudeEngine
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_DisableAttitudeEngine(void);

/**
 * @brief Configure accelerometer low-pass filter
 * 
 * @param enable Enable or disable LPF
 * @param mode LPF mode
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ConfigAccelLPF(bool enable, QMI8658C_LPFMode_t mode);

/**
 * @brief Configure gyroscope low-pass filter
 * 
 * @param enable Enable or disable LPF
 * @param mode LPF mode
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ConfigGyroLPF(bool enable, QMI8658C_LPFMode_t mode);

/**
 * @brief Configure FIFO
 * 
 * @param mode FIFO mode
 * @param size FIFO size
 * @param watermark FIFO watermark level
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ConfigFIFO(QMI8658C_FifoMode_t mode, QMI8658C_FifoSize_t size, uint8_t watermark);

/**
 * @brief Reset FIFO
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ResetFIFO(void);

/**
 * @brief Read data from FIFO
 * 
 * @param buffer Pointer to buffer to store FIFO data
 * @param count Pointer to variable to store number of bytes read
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ReadFIFO(uint8_t *buffer, uint16_t *count);

/**
 * @brief Read 6-axis IMU data (accelerometer + gyroscope)
 * 
 * @param data Pointer to structure to store IMU data
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ReadIMUData(QMI8658C_IMUData_t *data);

/**
 * @brief Read 9-axis data (accelerometer + gyroscope + magnetometer)
 * 
 * @param data Pointer to structure to store 9DOF data
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_Read9DOFData(QMI8658C_9DOFData_t *data);

/**
 * @brief Read AttitudeEngine data (quaternion and velocity increments)
 * 
 * @param data Pointer to structure to store AttitudeEngine data
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ReadAttitudeEngineData(QMI8658C_AttitudeEngine_t *data);

/**
 * @brief Configure Wake on Motion (WoM) feature
 * 
 * @param threshold Motion threshold in mg (0-255)
 * @param useINT2 Use INT2 instead of INT1 for WoM interrupts
 * @param initialValue Initial pin state
 * @param blankingTime Interrupt blanking time (in samples)
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ConfigWakeOnMotion(uint8_t threshold, bool useINT2, 
                                              bool initialValue, uint8_t blankingTime);

/**
 * @brief Disable Wake on Motion feature
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_DisableWakeOnMotion(void);

/**
 * @brief Perform accelerometer self-test
 * 
 * @param result Pointer to store self-test result (true for pass, false for fail)
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_AccelSelfTest(bool *result);

/**
 * @brief Perform gyroscope self-test
 * 
 * @param result Pointer to store self-test result (true for pass, false for fail)
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_GyroSelfTest(bool *result);

/**
 * @brief Set gyroscope bias
 * 
 * @param xBias X-axis bias
 * @param yBias Y-axis bias
 * @param zBias Z-axis bias
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_SetGyroBias(int16_t xBias, int16_t yBias, int16_t zBias);

/**
 * @brief Set accelerometer bias
 * 
 * @param xBias X-axis bias
 * @param yBias Y-axis bias
 * @param zBias Z-axis bias
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_SetAccelBias(int16_t xBias, int16_t yBias, int16_t zBias);

/**
 * @brief Enter power-down mode
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_EnterPowerDown(void);

/**
 * @brief Exit power-down mode
 * 
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ExitPowerDown(void);

/**
 * @brief Execute CTRL9 command and wait for completion
 * 
 * @param command Command to execute
 * @return QMI8658C_Status_t Operation status
 */
QMI8658C_Status_t QMI8658C_ExecuteCommand(uint8_t command);

#ifdef __cplusplus
}
#endif

#endif /* QMI8658C_H */
